# AutoBenchmark

Determine the speed of various AceSorting functions, for various array sizes.

**Version**: AceSorting v0.1

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## Dependencies

This program depends on the following libraries:

* [AceCommon](https://github.com/bxparks/AceCommon)

## How to Generate

This requires the [AUniter](https://github.com/bxparks/AUniter) script
to execute the Arduino IDE programmatically.

The `Makefile` has rules to generate the `*.txt` results file for several
microcontrollers that I usually support, but the `$ make benchmarks` command
does not work very well because the USB port of the microcontroller is a
dynamically changing parameter. I created a semi-automated way of collect the
`*.txt` files:

1. Connect the microcontroller to the serial port. I usually do this through a
USB hub with individually controlled switch.
2. Type `$ auniter ports` to determine its `/dev/ttyXXX` port number (e.g.
`/dev/ttyUSB0` or `/dev/ttyACM0`).
3. If the port is `USB0` or `ACM0`, type `$ make nano.txt`, etc.
4. Switch off the old microontroller.
5. Go to Step 1 and repeat for each microcontroller.

The `generate_table.awk` program reads one of `*.txt` files and prints out an
ASCII table that can be directly embedded into this README.md file. For example
the following command produces the table in the Nano section below:

```
$ ./generate_table.awk < nano.txt
```

Fortunately, we no longer need to run `generate_table.awk` for each `*.txt`
file. The process has been automated using the `generate_readme.py` script which
will be invoked by the following command:
```
$ make README.md
```

## CPU Time Changes

**v0.1**

* Initial results.
* The C-library `qsort()` is far slower than the C++ version because it uses
  a callback to a comparison function through a function pointer.

## Results

The following results show the runtime of each sorting function in milliseconds,
when sorting different array sizes.

### Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.13
* Arduino AVR Boards 1.8.3
* `micros()` has a resolution of 4 microseconds

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.101 | 0.989 | 11.841 | 112.304 |         |         |
| insertionSort()     | 0.045 | 0.275 |  2.601 |  21.651 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.090 | 0.365 |  1.797 |   7.412 |         |         |
| shellSortKnuth()    | 0.102 | 0.329 |  1.443 |   5.728 |         |         |
| shellSortTokuda()   | 0.074 | 0.340 |  1.631 |   6.554 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.163 | 0.550 |  2.220 |   8.135 |         |         |
| combSort13m()       | 0.164 | 0.551 |  2.238 |   8.141 |         |         |
| combSort133()       | 0.085 | 0.388 |  1.950 |   7.691 |         |         |
| combSort133m()      | 0.089 | 0.419 |  1.995 |   7.730 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.096 | 0.374 |  1.558 |   5.665 |         |         |
| quickSortMedian()   | 0.118 | 0.429 |  1.711 |   5.863 |         |         |
| quickSortMdnSwppd() | 0.091 | 0.334 |  1.399 |   4.893 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.203 | 0.863 |  3.663 |  13.016 |         |         |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### SparkFun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.13
* SparkFun AVR Boards 1.1.13
* `micros()` has a resolution of 4 microseconds

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.112 | 1.011 | 13.172 | 118.507 |         |         |
| insertionSort()     | 0.048 | 0.269 |  2.481 |  22.068 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.090 | 0.370 |  1.825 |   7.457 |         |         |
| shellSortKnuth()    | 0.100 | 0.333 |  1.437 |   5.730 |         |         |
| shellSortTokuda()   | 0.076 | 0.338 |  1.627 |   6.553 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.165 | 0.544 |  2.291 |   8.196 |         |         |
| combSort13m()       | 0.163 | 0.556 |  2.236 |   8.131 |         |         |
| combSort133()       | 0.087 | 0.396 |  1.982 |   7.762 |         |         |
| combSort133m()      | 0.088 | 0.422 |  1.985 |   7.759 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.096 | 0.367 |  1.566 |   5.604 |         |         |
| quickSortMedian()   | 0.120 | 0.424 |  1.722 |   5.928 |         |         |
| quickSortMdnSwppd() | 0.089 | 0.340 |  1.394 |   4.957 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.206 | 0.834 |  3.649 |  13.080 |         |         |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.13
* SparkFun SAMD Core 1.8.3

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.033 | 0.341 |  3.915 |  36.720 | 413.076 |         |
| insertionSort()     | 0.018 | 0.094 |  0.936 |   8.204 |  87.954 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.022 | 0.085 |  0.416 |   1.700 |   7.543 |  27.936 |
| shellSortKnuth()    | 0.019 | 0.066 |  0.293 |   1.150 |   5.078 |  19.490 |
| shellSortTokuda()   | 0.020 | 0.082 |  0.392 |   1.558 |   6.689 |  24.188 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.035 | 0.120 |  0.510 |   1.848 |   8.194 |  29.611 |
| combSort13m()       | 0.036 | 0.125 |  0.512 |   1.872 |   7.944 |  28.631 |
| combSort133()       | 0.021 | 0.091 |  0.448 |   1.754 |   7.939 |  27.346 |
| combSort133m()      | 0.022 | 0.099 |  0.456 |   1.752 |   7.621 |  26.992 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.021 | 0.075 |  0.311 |   1.065 |   4.237 |  14.362 |
| quickSortMedian()   | 0.026 | 0.087 |  0.348 |   1.167 |   4.402 |  14.555 |
| quickSortMdnSwppd() | 0.020 | 0.071 |  0.282 |   0.982 |   3.775 |  12.711 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.050 | 0.206 |  0.861 |   3.071 |  11.883 |  40.299 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### STM32

* STM32 "Blue Pill", STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.13
* STM32duino 2.0.0

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.019 | 0.200 |  2.056 |  19.174 | 229.211 |         |
| insertionSort()     | 0.009 | 0.053 |  0.512 |   4.584 |  49.112 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.018 | 0.074 |  0.363 |   1.492 |   6.411 |  23.660 |
| shellSortKnuth()    | 0.014 | 0.059 |  0.290 |   1.182 |   5.308 |  20.425 |
| shellSortTokuda()   | 0.011 | 0.048 |  0.228 |   0.915 |   3.888 |  14.116 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.019 | 0.090 |  0.452 |   1.794 |   8.182 |  30.001 |
| combSort13m()       | 0.020 | 0.091 |  0.446 |   1.715 |   7.666 |  27.604 |
| combSort133()       | 0.018 | 0.084 |  0.429 |   1.685 |   7.692 |  26.113 |
| combSort133m()      | 0.018 | 0.092 |  0.434 |   1.702 |   7.481 |  26.431 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.018 | 0.069 |  0.292 |   1.041 |   4.061 |  13.649 |
| quickSortMedian()   | 0.021 | 0.078 |  0.314 |   1.120 |   4.218 |  14.402 |
| quickSortMdnSwppd() | 0.014 | 0.054 |  0.232 |   0.831 |   3.249 |  11.240 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.032 | 0.134 |  0.573 |   2.124 |   8.337 |  28.772 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### ESP8266

* NodeMCU 1.0 clone, 80MHz ESP8266
* Arduino IDE 1.8.13
* ESP8266 Boards 2.7.4

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.021 | 0.191 |  2.232 |  20.144 | 225.651 |         |
| insertionSort()     | 0.009 | 0.037 |  0.362 |   3.220 |  34.646 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.013 | 0.050 |  0.246 |   1.020 |   4.402 |  16.259 |
| shellSortKnuth()    | 0.010 | 0.036 |  0.173 |   0.691 |   3.016 |  11.270 |
| shellSortTokuda()   | 0.009 | 0.038 |  0.181 |   0.730 |   3.141 |  11.368 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.013 | 0.049 |  0.229 |   0.836 |   3.810 |  13.769 |
| combSort13m()       | 0.013 | 0.051 |  0.225 |   0.842 |   3.621 |  13.130 |
| combSort133()       | 0.010 | 0.040 |  0.210 |   0.804 |   3.603 |  12.406 |
| combSort133m()      | 0.010 | 0.044 |  0.205 |   0.803 |   3.448 |  12.407 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.013 | 0.046 |  0.186 |   0.641 |   2.470 |   8.390 |
| quickSortMedian()   | 0.016 | 0.052 |  0.201 |   0.679 |   2.577 |   8.439 |
| quickSortMdnSwppd() | 0.012 | 0.039 |  0.158 |   0.550 |   2.116 |   7.180 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.027 | 0.092 |  0.414 |   1.501 |   6.002 |  20.681 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.13
* ESP32 Boards 1.0.6

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.006 | 0.052 |  0.624 |   5.731 |  63.186 |         |
| insertionSort()     | 0.004 | 0.016 |  0.159 |   1.403 |  14.734 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.005 | 0.017 |  0.079 |   0.326 |   1.441 |   5.334 |
| shellSortKnuth()    | 0.004 | 0.013 |  0.061 |   0.250 |   1.097 |   4.168 |
| shellSortTokuda()   | 0.004 | 0.014 |  0.065 |   0.256 |   1.111 |   4.031 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.004 | 0.019 |  0.094 |   0.376 |   1.677 |   6.256 |
| combSort13m()       | 0.005 | 0.021 |  0.098 |   0.381 |   1.651 |   5.984 |
| combSort133()       | 0.004 | 0.019 |  0.091 |   0.368 |   1.678 |   5.718 |
| combSort133m()      | 0.005 | 0.020 |  0.095 |   0.365 |   1.584 |   5.682 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.005 | 0.018 |  0.071 |   0.259 |   1.005 |   3.401 |
| quickSortMedian()   | 0.006 | 0.018 |  0.072 |   0.251 |   0.969 |   3.257 |
| quickSortMdnSwppd() | 0.004 | 0.014 |  0.059 |   0.205 |   0.817 |   2.796 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.008 | 0.032 |  0.144 |   0.525 |   2.109 |   7.245 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.13
* Teensyduino 1.53
* Compiler options: "Faster"

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.008 | 0.077 |  0.957 |   8.534 |  97.649 |         |
| insertionSort()     | 0.006 | 0.044 |  0.392 |   3.410 |  36.368 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.009 | 0.034 |  0.168 |   0.688 |   3.021 |  11.131 |
| shellSortKnuth()    | 0.006 | 0.027 |  0.131 |   0.524 |   2.326 |   8.970 |
| shellSortTokuda()   | 0.007 | 0.028 |  0.137 |   0.542 |   2.311 |   8.349 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.008 | 0.036 |  0.179 |   0.716 |   3.253 |  11.911 |
| combSort13m()       | 0.009 | 0.038 |  0.185 |   0.719 |   3.139 |  11.372 |
| combSort133()       | 0.008 | 0.035 |  0.170 |   0.695 |   3.169 |  10.965 |
| combSort133m()      | 0.008 | 0.038 |  0.180 |   0.702 |   3.016 |  10.884 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.007 | 0.026 |  0.110 |   0.392 |   1.541 |   5.275 |
| quickSortMedian()   | 0.009 | 0.030 |  0.122 |   0.414 |   1.605 |   5.358 |
| quickSortMdnSwppd() | 0.007 | 0.025 |  0.102 |   0.364 |   1.402 |   4.717 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.017 | 0.072 |  0.313 |   1.154 |   4.585 |  15.785 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

